@@ -16,6 +16,8 @@
 import base64
 import subprocess
 
+import six
+
 
 class DecryptionFailure(Exception):
     pass
@@ -35,4 +37,7 @@ def decrypt_password(private_key, password):
     proc.stdin.close()
     if proc.returncode:
         raise DecryptionFailure(err)
+
+    if not six.PY2 and isinstance(out, bytes):
+        return out.decode('utf-8')
     return out
