@@ -29,8 +29,6 @@ from oslo_utils import importutils
 from oslo_utils import strutils
 import six
 
-osprofiler_profiler = importutils.try_import("osprofiler.profiler")
-
 import novaclient
 from novaclient import api_versions
 from novaclient import client
@@ -39,6 +37,8 @@ import novaclient.extension
 from novaclient.i18n import _
 from novaclient import utils
 
+osprofiler_profiler = importutils.try_import("osprofiler.profiler")
+
 DEFAULT_MAJOR_OS_COMPUTE_API_VERSION = "2.0"
 # The default behaviour of nova client CLI is that CLI negotiates with server
 # to find out the most recent version between client and server, and
@@ -270,6 +270,14 @@ class OpenStackComputeShell(object):
             'OS_PROJECT_NAME', 'OS_TENANT_NAME', 'NOVA_PROJECT_ID'))
         parser.set_defaults(os_project_id=utils.env(
             'OS_PROJECT_ID', 'OS_TENANT_ID'))
+        parser.set_defaults(
+            os_project_domain_id=utils.env('OS_PROJECT_DOMAIN_ID'))
+        parser.set_defaults(
+            os_project_domain_name=utils.env('OS_PROJECT_DOMAIN_NAME'))
+        parser.set_defaults(
+            os_user_domain_id=utils.env('OS_USER_DOMAIN_ID'))
+        parser.set_defaults(
+            os_user_domain_name=utils.env('OS_USER_DOMAIN_NAME'))
 
     def get_base_parser(self, argv):
         parser = NovaClientArgumentParser(
@@ -350,19 +358,32 @@ class OpenStackComputeShell(object):
                    '"X.latest", defaults to env[OS_COMPUTE_API_VERSION].'))
 
         parser.add_argument(
-            '--endpoint-override',
+            '--os-endpoint-override',
             metavar='<bypass-url>',
             dest='endpoint_override',
-            default=utils.env('NOVACLIENT_ENDPOINT_OVERRIDE',
+            default=utils.env('OS_ENDPOINT_OVERRIDE',
+                              'NOVACLIENT_ENDPOINT_OVERRIDE',
                               'NOVACLIENT_BYPASS_URL'),
             help=_("Use this API endpoint instead of the Service Catalog. "
-                   "Defaults to env[NOVACLIENT_ENDPOINT_OVERRIDE]."))
+                   "Defaults to env[OS_ENDPOINT_OVERRIDE]."))
+
+        # NOTE(takashin): This dummy '--end' argument was added
+        # to avoid misinterpreting command line arguments.
+        # If there is not this dummy argument, the '--end' is interpreted to
+        # the '--endpoint-override'.
+        # TODO(takashin): Remove this dummy '--end' argument
+        # when the deprecated '--endpoint-override' argument is removed.
+        parser.add_argument(
+            '--end',
+            metavar='<end>',
+            nargs='?',
+            help=argparse.SUPPRESS)
 
         parser.add_argument(
-            '--bypass-url',
+            '--endpoint-override',
             action=DeprecatedAction,
-            use=_('use "%s"; this option will be removed after Pike OpenStack '
-                  'release.') % '--os-endpoint-override',
+            use=_('use "%s"; this option will be removed after Rocky '
+                  'OpenStack release.') % '--os-endpoint-override',
             dest='endpoint_override',
             help=argparse.SUPPRESS)
 
@@ -596,6 +617,26 @@ class OpenStackComputeShell(object):
                     _("You must provide an auth url "
                       "via either --os-auth-url or env[OS_AUTH_URL]."))
 
+            # TODO(Shilpasd): need to provide support in python - novaclient
+            # for required options for below default auth type plugins:
+            # 1. v3oidcclientcredential
+            # 2. v3oidcpassword
+            # 3. v3oidcauthcode
+            # 4. v3oidcaccesstoken
+            # 5. v3oauth1
+            # 6. v3fedkerb
+            # 7. v3adfspassword
+            # 8. v3samlpassword
+            # 9. v3applicationcredential
+            # TODO(Shilpasd): need to provide support in python - novaclient
+            # for below extra keystoneauth auth type plugins:
+            # We will need to add code to support discovering of versions
+            # supported by the keystone service based on the auth_url similar
+            # to the one supported by glanceclient.
+            # 1. v3password
+            # 2. v3token
+            # 3. v3kerberos
+            # 4. v3totp
             with utils.record_time(self.times, args.timings,
                                    'auth_url', args.os_auth_url):
                 keystone_session = (
