@@ -453,6 +453,12 @@ class ServersTest(utils.FixturedTestCase):
         self.assert_request_id(server, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('GET', '/servers/1234')
         self.assertEqual('sample-server', server.name)
+        # The networks should be sorted.
+        networks = server.networks
+        self.assertEqual(2, len(networks))
+        labels = list(networks)  # returns the dict keys
+        self.assertEqual('private', labels[0])
+        self.assertEqual('public', labels[1])
 
         self.assertRaises(exceptions.NoUniqueMatch, self.cs.servers.find,
                           flavor={"id": 1, "name": "256 MiB Server"})
@@ -941,6 +947,7 @@ class ServersTest(utils.FixturedTestCase):
         ret = s.interface_attach(None, None, None)
         self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
         self.assert_called('POST', '/servers/1234/os-interface')
+        self.assertIsInstance(ret, servers.NetworkInterface)
 
     def test_interface_detach(self):
         s = self.cs.servers.get(1234)
@@ -1409,6 +1416,7 @@ class ServersV249Test(ServersV2_37Test):
             {'interfaceAttachment':
                 {'port_id': '7f42712e-63fe-484c-a6df-30ae4867ff66',
                  'tag': 'test_tag'}})
+        self.assertIsInstance(ret, servers.NetworkInterface)
 
     def test_add_fixed_ip(self):
         # novaclient.v2.servers.Server.add_fixed_ip()
@@ -1655,3 +1663,43 @@ class ServersV267Test(ServersV263Test):
                                nics='none', block_device_mapping_v2=bdm)
         self.assertIn("Block device volume_type is not supported before "
                       "microversion 2.67", six.text_type(ex))
+
+
+class ServersV268Test(ServersV267Test):
+
+    api_version = "2.68"
+
+    def test_evacuate(self):
+        s = self.cs.servers.get(1234)
+        ret = s.evacuate('fake_target_host')
+        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action',
+                           {'evacuate': {'host': 'fake_target_host'}})
+
+        ret = self.cs.servers.evacuate(s, 'fake_target_host')
+        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action',
+                           {'evacuate': {'host': 'fake_target_host'}})
+
+        ex = self.assertRaises(TypeError, self.cs.servers.evacuate,
+                               'fake_target_host', force=True)
+        self.assertIn('force', six.text_type(ex))
+
+    def test_live_migrate_server(self):
+        s = self.cs.servers.get(1234)
+        ret = s.live_migrate(host='hostname', block_migration='auto')
+        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action',
+                           {'os-migrateLive': {'host': 'hostname',
+                                               'block_migration': 'auto'}})
+
+        ret = self.cs.servers.live_migrate(s, host='hostname',
+                                           block_migration='auto')
+        self.assert_request_id(ret, fakes.FAKE_REQUEST_ID_LIST)
+        self.assert_called('POST', '/servers/1234/action',
+                           {'os-migrateLive': {'host': 'hostname',
+                                               'block_migration': 'auto'}})
+
+        ex = self.assertRaises(TypeError, self.cs.servers.live_migrate,
+                               host='hostname', force=True)
+        self.assertIn('force', six.text_type(ex))
